@@ -30,6 +30,9 @@
 #include "gda-enums.h"
 #include "gda-data-select.h"
 
+extern GdaAttributesManager *gda_column_attributes_manager;
+extern GdaAttributesManager *gda_holder_attributes_manager;
+
 /* 
  * Main static functions 
  */
@@ -50,9 +53,10 @@ static void gda_data_model_iter_get_property (GObject *object,
 /* follow model changes */
 static void model_row_updated_cb (GdaDataModel *model, gint row, GdaDataModelIter *iter);
 static void model_row_removed_cb (GdaDataModel *model, gint row, GdaDataModelIter *iter);
+static void model_reset_cb (GdaDataModel *model, GdaDataModelIter *iter);
 
 static GError *validate_holder_change_cb (GdaSet *paramlist, GdaHolder *param, const GValue *new_value);
-static void holder_attr_changed_cb (GdaSet *paramlist, GdaHolder *param);
+static void holder_attr_changed_cb (GdaSet *paramlist, GdaHolder *param, const gchar *att_name, const GValue *att_value);
 
 /* get a pointer to the parents to be able to cvalue their destructor */
 static GObjectClass  *parent_class = NULL;
@@ -81,7 +85,7 @@ enum
 struct _GdaDataModelIterPrivate
 {
 	GdaDataModel          *data_model;
-	gulong                 model_changes_signals[2];
+	gulong                 model_changes_signals[3];
 	gboolean               keep_param_changes;
 	gint                   row; /* -1 if row is unknown */
 };
@@ -185,6 +189,7 @@ gda_data_model_iter_init (GdaDataModelIter *iter)
 	iter->priv->row = -1;
 	iter->priv->model_changes_signals[0] = 0;
 	iter->priv->model_changes_signals[1] = 0;
+	iter->priv->model_changes_signals[2] = 0;
 	iter->priv->keep_param_changes = FALSE;
 }
 
@@ -221,6 +226,30 @@ model_row_removed_cb (GdaDataModel *model, gint row, GdaDataModelIter *iter)
 	}
 }
 
+static void
+model_reset_cb (GdaDataModel *model, GdaDataModelIter *iter)
+{
+	/* reset the iter to before the 1st row */
+	gda_data_model_iter_invalidate_contents (iter);
+	gda_data_model_iter_move_at_row (iter, -1);
+	
+	/* adjust GdaHolder's type if a column's type has changed from GDA_TYPE_NULL
+	 * to something else */
+	gint i;
+	GSList *list;
+	for (i = 0, list = ((GdaSet*) iter)->holders;
+	     list;
+	     i++, list = list->next) {
+		if (gda_holder_get_g_type ((GdaHolder *) list->data) == GDA_TYPE_NULL) {
+			GdaColumn *col;
+			col = gda_data_model_describe_column (model, i);
+			if (gda_column_get_g_type (col) != GDA_TYPE_NULL)
+				g_object_set (G_OBJECT (list->data), "g-type",
+					      gda_column_get_g_type (col), NULL);
+		}
+	}
+}
+
 /*
  * This function is called when a parameter in @paramlist is changed
  * to make sure the change is propagated to the GdaDataModel
@@ -252,7 +281,7 @@ validate_holder_change_cb (GdaSet *paramlist, GdaHolder *param, const GValue *ne
 			}
 		}
 		else if (! gda_data_model_set_value_at ((GdaDataModel *) iter->priv->data_model, 
-							col, iter->priv->row, gda_holder_get_value (param), &error)) {
+							col, iter->priv->row, new_value, &error)) {
 			if (!error)
 				g_set_error (&error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
 					     _("GdaDataModel refused value change"));
@@ -275,17 +304,18 @@ validate_holder_change_cb (GdaSet *paramlist, GdaHolder *param, const GValue *ne
  * paramlist is an iter for
  */
 static void
-holder_attr_changed_cb (GdaSet *paramlist, GdaHolder *param)
+holder_attr_changed_cb (GdaSet *paramlist, GdaHolder *param, const gchar *att_name, const GValue *att_value)
 {
 	GdaDataModelIter *iter;
 	gint col;
-	gboolean toset;
+	gboolean toset = FALSE;
 
 	iter = GDA_DATA_MODEL_ITER (paramlist);
 	if (!GDA_IS_DATA_PROXY (iter->priv->data_model))
 		return;
 
-	if (!iter->priv->keep_param_changes && (iter->priv->row >= 0)) {
+	if (!strcmp (att_name, GDA_ATTRIBUTE_IS_DEFAULT) &&
+	    !iter->priv->keep_param_changes && (iter->priv->row >= 0)) {
 		g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [0]);
 		g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [1]);
 		
@@ -293,7 +323,8 @@ holder_attr_changed_cb (GdaSet *paramlist, GdaHolder *param)
 		col = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "model_col")) - 1;
 		g_return_if_fail (col >= 0);
 		
-		g_object_get (G_OBJECT (param), "use-default-value", &toset, NULL);
+		if (att_value && g_value_get_boolean (att_value))
+			toset = TRUE;
 		if (toset && gda_holder_get_default_value (param))
 			gda_data_proxy_alter_value_attributes (GDA_DATA_PROXY (iter->priv->data_model), 
 							       iter->priv->row, col, 
@@ -305,7 +336,7 @@ holder_attr_changed_cb (GdaSet *paramlist, GdaHolder *param)
 
 	/* for the parent class */
 	if (((GdaSetClass *) parent_class)->holder_attr_changed)
-		((GdaSetClass *) parent_class)->holder_attr_changed (paramlist, param);
+		((GdaSetClass *) parent_class)->holder_attr_changed (paramlist, param, att_name, att_value);
 }
 
 static void
@@ -321,6 +352,7 @@ gda_data_model_iter_dispose (GObject *object)
 		if (iter->priv->data_model) { 
 			g_signal_handler_disconnect (iter->priv->data_model, iter->priv->model_changes_signals [0]);
 			g_signal_handler_disconnect (iter->priv->data_model, iter->priv->model_changes_signals [1]);
+			g_signal_handler_disconnect (iter->priv->data_model, iter->priv->model_changes_signals [2]);
 			g_object_remove_weak_pointer (G_OBJECT (iter->priv->data_model), 
 						      (gpointer*) &(iter->priv->data_model));
 			iter->priv->data_model = NULL;
@@ -376,10 +408,12 @@ set_holders_properties_from_select_stmt (GdaDataModelIter *iter, GdaConnection *
 		goto out;
 	}
 
+	/* FIXME: also set some column attributes using gda_column_set_attribute() */
+
 	GSList *fields, *holders;
 	for (fields = select->expr_list, holders = GDA_SET (iter)->holders; 
 	     fields && holders; 
-	     fields = fields->next, holders = holders->next) {
+	     fields = fields->next) {
 		GdaSqlSelectField *selfield = (GdaSqlSelectField*) fields->data;
 		if (selfield->validity_meta_table_column) {
 			GdaMetaTableColumn *tcol = selfield->validity_meta_table_column;
@@ -392,8 +426,10 @@ set_holders_properties_from_select_stmt (GdaDataModelIter *iter, GdaConnection *
 				gda_holder_set_default_value (GDA_HOLDER (holders->data), dvalue);
 				gda_value_free (dvalue);
 			}
+			holders = holders->next;
 		}
-		else if (selfield->validity_meta_object && (selfield->validity_meta_object->obj_type == GDA_META_DB_TABLE) &&
+		else if (selfield->validity_meta_object && 
+			 (selfield->validity_meta_object->obj_type == GDA_META_DB_TABLE) &&
 			 selfield->expr && selfield->expr->value && !selfield->expr->param_spec && 
 			 (G_VALUE_TYPE (selfield->expr->value) == G_TYPE_STRING) &&
 			 !strcmp (g_value_get_string (selfield->expr->value), "*")) {
@@ -410,10 +446,12 @@ set_holders_properties_from_select_stmt (GdaDataModelIter *iter, GdaConnection *
 					gda_holder_set_default_value (GDA_HOLDER (holders->data), dvalue);
 					gda_value_free (dvalue);
 				}
-				if (tmplist != mtable->columns)
+				if (tmplist)
 					holders = holders->next;
 			}
 		}
+		else
+			holders = holders->next;
 	}
 	if (fields || holders)
 		g_warning ("Internal error: GdaDataModelIter has %d GdaHolders, and SELECT statement has %d expressions",
@@ -464,7 +502,7 @@ gda_data_model_iter_set_property (GObject *object,
 				}
 				else {
 					const gchar *cstr;
-					cstr = gda_column_get_title (column);
+					cstr = gda_column_get_description (column);
 					if (!cstr)
 						cstr = gda_column_get_name (column);
 					if (cstr)
@@ -482,6 +520,9 @@ gda_data_model_iter_set_property (GObject *object,
 					gda_holder_set_default_value (param, v);
 					gda_value_free (v);
 				}
+				/* copy extra attributes */
+				gda_attributes_manager_copy (gda_column_attributes_manager, (gpointer) column,
+							     gda_holder_attributes_manager, (gpointer) param);
 				gda_set_add_holder ((GdaSet *) iter, param);
 				g_object_set_data (G_OBJECT (param), "model_col", GINT_TO_POINTER (col + 1));
 				g_object_unref (param);
@@ -496,6 +537,7 @@ gda_data_model_iter_set_property (GObject *object,
 					return;
 				g_signal_handler_disconnect (iter->priv->data_model, iter->priv->model_changes_signals [0]);
 				g_signal_handler_disconnect (iter->priv->data_model, iter->priv->model_changes_signals [1]);
+				g_signal_handler_disconnect (iter->priv->data_model, iter->priv->model_changes_signals [2]);
 				g_object_remove_weak_pointer (G_OBJECT (iter->priv->data_model), 
 							      (gpointer*) &(iter->priv->data_model)); 	
 			}
@@ -507,6 +549,8 @@ gda_data_model_iter_set_property (GObject *object,
 										  G_CALLBACK (model_row_updated_cb), iter);
 			iter->priv->model_changes_signals [1] = g_signal_connect (G_OBJECT (ptr), "row-removed",
 										  G_CALLBACK (model_row_removed_cb), iter);
+			iter->priv->model_changes_signals [2] = g_signal_connect (G_OBJECT (ptr), "reset",
+										  G_CALLBACK (model_reset_cb), iter);
 
 			if (GDA_IS_DATA_SELECT (iter->priv->data_model)) {
 				GdaStatement *sel_stmt;
