@@ -1,5 +1,5 @@
 /* GDA common library
- * Copyright (C) 1998 - 2007 The GNOME Foundation.
+ * Copyright (C) 1998 - 2008 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -27,7 +27,6 @@
 #include <glib-object.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
-#include <libgda/gda-command.h>
 #include <libgda/gda-decl.h>
 #include <libgda/gda-column.h>
 #include <libgda/gda-value.h>
@@ -40,10 +39,15 @@ G_BEGIN_DECLS
 #define GDA_IS_DATA_MODEL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, GDA_TYPE_DATA_MODEL))
 #define GDA_DATA_MODEL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GDA_TYPE_DATA_MODEL, GdaDataModelClass))
 
+/* error reporting */
+extern GQuark gda_data_model_error_quark (void);
+#define GDA_DATA_MODEL_ERROR gda_data_model_error_quark ()
+
 typedef enum {
 	GDA_DATA_MODEL_ACCESS_RANDOM = 1 << 0,
 	GDA_DATA_MODEL_ACCESS_CURSOR_FORWARD = 1 << 1,
 	GDA_DATA_MODEL_ACCESS_CURSOR_BACKWARD = 1 << 2,
+	GDA_DATA_MODEL_ACCESS_CURSOR = GDA_DATA_MODEL_ACCESS_CURSOR_FORWARD | GDA_DATA_MODEL_ACCESS_CURSOR_BACKWARD,
 	GDA_DATA_MODEL_ACCESS_INSERT  = 1 << 3,
 	GDA_DATA_MODEL_ACCESS_UPDATE  = 1 << 4,
 	GDA_DATA_MODEL_ACCESS_DELETE  = 1 << 5,
@@ -66,7 +70,10 @@ enum {
 	GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
 	GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
 	GDA_DATA_MODEL_VALUES_LIST_ERROR,
-	GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR
+	GDA_DATA_MODEL_VALUE_TYPE_ERROR,
+	GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
+	GDA_DATA_MODEL_ACCESS_ERROR,
+	GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR
 };
 
 /* struct for the interface */
@@ -80,7 +87,7 @@ struct _GdaDataModelClass {
 	GdaColumn           *(* i_describe_column)  (GdaDataModel *model, gint col);
 	GdaDataModelAccessFlags (* i_get_access_flags) (GdaDataModel *model);
 
-	const GValue        *(* i_get_value_at)     (GdaDataModel *model, gint col, gint row);
+	const GValue        *(* i_get_value_at)     (GdaDataModel *model, gint col, gint row, GError **error);
 	GdaValueAttribute    (* i_get_attributes_at)(GdaDataModel *model, gint col, gint row);
 	GdaDataModelIter    *(* i_create_iter)      (GdaDataModel *model);
 	gboolean             (* i_iter_at_row)      (GdaDataModel *model, GdaDataModelIter *iter, gint row);
@@ -89,6 +96,8 @@ struct _GdaDataModelClass {
 
 	gboolean             (* i_set_value_at)     (GdaDataModel *model, gint col, gint row, 
 						     const GValue *value, GError **error);
+	gboolean             (* i_iter_set_value)   (GdaDataModel *model, GdaDataModelIter *iter, gint col,
+						     const GValue *value, GError **error);
 	gboolean             (* i_set_values)       (GdaDataModel *model, gint row, GList *values,
 						     GError **error);
 	gint                 (* i_append_values)    (GdaDataModel *model, const GList *values, GError **error);
@@ -99,40 +108,32 @@ struct _GdaDataModelClass {
 	void                 (* i_set_notify)       (GdaDataModel *model, gboolean do_notify_changes);
 	gboolean             (* i_get_notify)       (GdaDataModel *model);
 	void                 (* i_send_hint)        (GdaDataModel *model, GdaDataModelHint hint, const GValue *hint_value);
-	gpointer                reserved1;
-	gpointer                reserved2;
-	gpointer                reserved3;
-	gpointer                reserved4;
 
 	/* signals */
 	void                 (* row_inserted)       (GdaDataModel *model, gint row);
 	void                 (* row_updated)        (GdaDataModel *model, gint row);
 	void                 (* row_removed)        (GdaDataModel *model, gint row);
+	void                 (* changed)            (GdaDataModel *model);
 	void                 (* reset)              (GdaDataModel *model);
-	gpointer                sig_reserved1;
-	gpointer                sig_reserved2;
-	gpointer                sig_reserved3;
-	gpointer                sig_reserved4;
 };
 
 GType               gda_data_model_get_type               (void) G_GNUC_CONST;
 
-gboolean            gda_data_model_is_updatable           (GdaDataModel *model);
-GdaDataModelAccessFlags gda_data_model_get_access_flags       (GdaDataModel *model);
+GdaDataModelAccessFlags gda_data_model_get_access_flags   (GdaDataModel *model);
 
 gint                gda_data_model_get_n_rows             (GdaDataModel *model);
 gint                gda_data_model_get_n_columns          (GdaDataModel *model);
 
 GdaColumn          *gda_data_model_describe_column        (GdaDataModel *model, gint col);
-gint                gda_data_model_get_column_index_by_name(GdaDataModel *model, const gchar *name);
+gint                gda_data_model_get_column_index       (GdaDataModel *model, const gchar *name);
 const gchar        *gda_data_model_get_column_name       (GdaDataModel *model, gint col);
 void                gda_data_model_set_column_name       (GdaDataModel *model, gint col, const gchar *name);
 const gchar        *gda_data_model_get_column_title       (GdaDataModel *model, gint col);
 void                gda_data_model_set_column_title       (GdaDataModel *model, gint col, const gchar *title);
 
-const GValue       *gda_data_model_get_value_at           (GdaDataModel *model, gint col, gint row);
-const GValue       *gda_data_model_get_value_at_col_name  (GdaDataModel *model, 
-							   const gchar *column_name, gint row);
+const GValue       *gda_data_model_get_value_at           (GdaDataModel *model, gint col, gint row, GError **error);
+const GValue       *gda_data_model_get_typed_value_at     (GdaDataModel *model, gint col, gint row, 
+							   GType expected_type, gboolean nullok, GError **error);
 GdaValueAttribute   gda_data_model_get_attributes_at      (GdaDataModel *model, gint col, gint row);
 GdaDataModelIter   *gda_data_model_create_iter            (GdaDataModel *model);
 void                gda_data_model_freeze                 (GdaDataModel *model);
@@ -151,21 +152,21 @@ void                gda_data_model_send_hint              (GdaDataModel *model,
 /* contents saving and loading */
 gchar              *gda_data_model_export_to_string       (GdaDataModel *model, GdaDataModelIOFormat format, 
 							   const gint *cols, gint nb_cols, 
-							   const gint *rows, gint nb_rows, GdaParameterList *options);
+							   const gint *rows, gint nb_rows, GdaSet *options);
 gboolean            gda_data_model_export_to_file         (GdaDataModel *model, GdaDataModelIOFormat format, 
 							   const gchar *file,
 							   const gint *cols, gint nb_cols, 
 							   const gint *rows, gint nb_rows, 
-							   GdaParameterList *options, GError **error);
+							   GdaSet *options, GError **error);
 
 gboolean            gda_data_model_import_from_model      (GdaDataModel *to, GdaDataModel *from, gboolean overwrite,
 							   GHashTable *cols_trans, GError **error);
 gboolean            gda_data_model_import_from_string     (GdaDataModel *model,
 							   const gchar *string, GHashTable *cols_trans,
-							   GdaParameterList *options, GError **error);
+							   GdaSet *options, GError **error);
 gboolean            gda_data_model_import_from_file       (GdaDataModel *model,
 							   const gchar *file, GHashTable *cols_trans,
-							   GdaParameterList *options, GError **error);
+							   GdaSet *options, GError **error);
 
 /* debug functions */
 void                gda_data_model_dump                   (GdaDataModel *model, FILE *to_stream);
