@@ -28,7 +28,8 @@
 #include "gda-marshal.h"
 #include "gda-util.h"
 #include <libgda.h>
-	
+#include <libgda/gda-attributes-manager.h>
+
 /* 
  * Main static functions 
  */
@@ -51,6 +52,7 @@ static void gda_holder_set_full_bind (GdaHolder *holder, GdaHolder *alias_of);
 
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass  *parent_class = NULL;
+GdaAttributesManager *gda_holder_attributes_manager;
 
 /* signals */
 enum
@@ -58,10 +60,11 @@ enum
 	CHANGED,
         SOURCE_CHANGED,
 	VALIDATE_CHANGE,
+	ATT_CHANGED,
         LAST_SIGNAL
 };
 
-static gint gda_holder_signals[LAST_SIGNAL] = { 0, 0, 0 };
+static gint gda_holder_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
 
 
 /* properties */
@@ -71,7 +74,6 @@ enum
 	PROP_ID,
 	PROP_NAME,
 	PROP_DESCR,
-	PROP_PLUGIN,
 	PROP_SIMPLE_BIND,
 	PROP_FULL_BIND,
 	PROP_SOURCE_MODEL,
@@ -84,8 +86,6 @@ enum
 struct _GdaHolderPrivate
 {
 	gchar           *id;
-	gchar           *name;
-	gchar           *descr;
 
 	GType            g_type;
 	GdaHolder       *full_bind;     /* FULL bind to holder */
@@ -93,6 +93,7 @@ struct _GdaHolderPrivate
 	
 	gboolean         invalid_forced;
 	gboolean         valid;
+	gboolean         is_freeable;
 
 	GValue           *value;
 	GValue          *default_value; /* CAN be either NULL or of any type */
@@ -101,8 +102,6 @@ struct _GdaHolderPrivate
 
 	GdaDataModel    *source_model;
 	gint             source_col;
-
-	gchar           *plugin;        /* plugin to be used for user interaction */
 };
 
 /* module error */
@@ -162,6 +161,12 @@ m_validate_change (GdaHolder *holder, const GValue *new_value)
 	return NULL;
 }
 
+static void
+holder_attribute_set_cb (GObject *obj, const gchar *att_name, const GValue *value, gpointer data)
+{
+	g_signal_emit (obj, gda_holder_signals[ATT_CHANGED], 0, att_name, value);
+}
+
 static void
 gda_holder_class_init (GdaHolderClass *class)
 {
@@ -183,6 +188,15 @@ gda_holder_class_init (GdaHolderClass *class)
                               G_STRUCT_OFFSET (GdaHolderClass, changed),
                               NULL, NULL,
                               gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
+	gda_holder_signals[ATT_CHANGED] =
+                g_signal_new ("attribute-changed",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdaHolderClass, att_changed),
+                              NULL, NULL,
+                              gda_marshal_VOID__STRING_POINTER, G_TYPE_NONE, 2, 
+			      G_TYPE_STRING, G_TYPE_POINTER);
+
 	/**
 	 * GdaHolder::before-change:
 	 * @holder: the object which received the signal
@@ -205,6 +219,7 @@ gda_holder_class_init (GdaHolderClass *class)
         class->changed = NULL;
         class->source_changed = NULL;
         class->validate_change = m_validate_change;
+	class->att_changed = NULL;
 
 	/* virtual functions */
 	object_class->dispose = gda_holder_dispose;
@@ -222,14 +237,9 @@ gda_holder_class_init (GdaHolderClass *class)
 	g_object_class_install_property (object_class, PROP_DESCR,
 					 g_param_spec_string ("description", NULL, NULL, NULL, 
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
-        g_object_class_install_property (object_class, PROP_GDA_TYPE,
-                                         g_param_spec_ulong ("g-type", NULL, NULL,
-							   0, G_MAXULONG, GDA_TYPE_NULL,
-							   (G_PARAM_READABLE | 
-							    G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY)));
-	g_object_class_install_property (object_class, PROP_PLUGIN,
-					 g_param_spec_string ("plugin", NULL, NULL, NULL, 
-							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+	g_object_class_install_property (object_class, PROP_GDA_TYPE,
+					 g_param_spec_gtype ("g-type", NULL, NULL, G_TYPE_NONE, 
+							     (G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT)));
 	g_object_class_install_property (object_class, PROP_NOT_NULL,
 					 g_param_spec_boolean ("not-null", NULL, NULL, FALSE,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
@@ -249,6 +259,9 @@ gda_holder_class_init (GdaHolderClass *class)
                                          g_param_spec_int ("source-column", NULL, NULL,
 							   0, G_MAXINT, 0,
 							   (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+	
+	/* extra */
+	gda_holder_attributes_manager = gda_attributes_manager_new (TRUE, holder_attribute_set_cb, NULL);
 }
 
 static void
@@ -257,8 +270,6 @@ gda_holder_init (GdaHolder *holder)
 	holder->priv = g_new0 (GdaHolderPrivate, 1);
 
 	holder->priv->id = NULL;
-	holder->priv->name = NULL;
-	holder->priv->descr = NULL;
 
 	holder->priv->g_type = G_TYPE_INVALID;
 	holder->priv->full_bind = NULL;
@@ -267,13 +278,13 @@ gda_holder_init (GdaHolder *holder)
 	holder->priv->invalid_forced = FALSE;
 	holder->priv->valid = TRUE;
 	holder->priv->default_forced = FALSE;
+	holder->priv->is_freeable = TRUE;
 	holder->priv->value = NULL;
 	holder->priv->default_value = NULL;
 
 	holder->priv->not_null = FALSE;
 	holder->priv->source_model = NULL;
 	holder->priv->source_col = 0;
-	holder->priv->plugin = NULL;
 }
 
 /**
@@ -287,13 +298,7 @@ gda_holder_init (GdaHolder *holder)
 GdaHolder *
 gda_holder_new (GType type)
 {
-	GObject   *obj;
-
-	g_return_val_if_fail (type != G_TYPE_INVALID, NULL);
-
-        obj = g_object_new (GDA_TYPE_HOLDER, "g-type", type, NULL);
-
-        return (GdaHolder *) obj;
+	return (GdaHolder*) g_object_new (GDA_TYPE_HOLDER, "g-type", type, NULL);
 }
 
 /**
@@ -301,6 +306,9 @@ gda_holder_new (GType type)
  * @orig: a #GdaHolder object to copy
  *
  * Copy constructor.
+ * 
+ * Note1: if @orig is set with a static value (see #gda_holder_take_static_value ()) 
+ * its copy will have a fresh new allocated GValue, so that user should free it when done.
  *
  * Returns: a new #GdaHolder object
  */
@@ -319,10 +327,6 @@ gda_holder_copy (GdaHolder *orig)
 
 	if (orig->priv->id)
 		holder->priv->id = g_strdup (orig->priv->id);
-	if (orig->priv->name)
-		holder->priv->name = g_strdup (orig->priv->name);
-	if (orig->priv->descr)
-		holder->priv->descr = g_strdup (orig->priv->descr);
 
 	if (orig->priv->full_bind)
 		gda_holder_set_full_bind (holder, orig->priv->full_bind);
@@ -339,14 +343,21 @@ gda_holder_copy (GdaHolder *orig)
 		/* direct settings */
 		holder->priv->invalid_forced = orig->priv->invalid_forced;
 		holder->priv->valid = orig->priv->valid;
+		holder->priv->is_freeable = TRUE;
 		holder->priv->default_forced = orig->priv->default_forced;	
 		if (orig->priv->value)
 			holder->priv->value = gda_value_copy (orig->priv->value);
 		if (orig->priv->default_value)
 			holder->priv->default_value = gda_value_copy (orig->priv->default_value);
 		holder->priv->not_null = orig->priv->not_null;
-		if (orig->priv->plugin)
-			holder->priv->plugin = g_strdup (orig->priv->plugin);
+		gda_attributes_manager_copy (gda_holder_attributes_manager, (gpointer) orig, gda_holder_attributes_manager, (gpointer) holder);
+
+		GValue *att_value;
+		g_value_set_boolean ((att_value = gda_value_new (G_TYPE_BOOLEAN)), holder->priv->default_forced);
+		gda_holder_set_attribute (holder, GDA_ATTRIBUTE_IS_DEFAULT, att_value);
+		gda_value_free (att_value);
+
+
 		return holder;
 	}
 	else {
@@ -423,6 +434,12 @@ gda_holder_new_inline (GType type, const gchar *id, ...)
 			g_value_set_float (value, va_arg (ap, double));
 		else if (type == G_TYPE_DOUBLE)
 			g_value_set_double (value, va_arg (ap, gdouble));
+		else if (type == G_TYPE_GTYPE)
+			g_value_set_gtype (value, va_arg (ap, GType));
+		else if (type == G_TYPE_LONG)
+			g_value_set_long (value, va_arg (ap, glong));
+		else if (type == G_TYPE_ULONG)
+			g_value_set_ulong (value, va_arg (ap, gulong));
 		else if (type == GDA_TYPE_NUMERIC)
 			gda_value_set_numeric (value, va_arg (ap, GdaNumeric *));
 		else if (type == G_TYPE_DATE)
@@ -466,8 +483,9 @@ gda_holder_dispose (GObject *object)
 
 		holder->priv->g_type = G_TYPE_INVALID;
 
-		if (holder->priv->value) {
-			gda_value_free (holder->priv->value);
+		if (holder->priv->value) {			
+			if (holder->priv->is_freeable)
+				gda_value_free (holder->priv->value);
 			holder->priv->value = NULL;
 		}
 
@@ -492,9 +510,6 @@ gda_holder_finalize (GObject   * object)
 	holder = GDA_HOLDER (object);
 	if (holder->priv) {
 		g_free (holder->priv->id);
-		g_free (holder->priv->name);
-		g_free (holder->priv->descr);
-		g_free (holder->priv->plugin);
 
 		g_free (holder->priv);
 		holder->priv = NULL;
@@ -511,7 +526,6 @@ gda_holder_set_property (GObject *object,
 			 const GValue *value,
 			 GParamSpec *pspec)
 {
-	const gchar *ptr;
 	GdaHolder *holder;
 
 	holder = GDA_HOLDER (object);
@@ -522,24 +536,16 @@ gda_holder_set_property (GObject *object,
 			holder->priv->id = g_value_dup_string (value);
 			break;
 		case PROP_NAME:
-			g_free (holder->priv->name);
-			holder->priv->name = g_value_dup_string (value);
+			gda_holder_set_attribute (holder, GDA_ATTRIBUTE_NAME, value);
 			break;
 		case PROP_DESCR:
-			g_free (holder->priv->descr);
-			holder->priv->descr = g_value_dup_string (value);
+			gda_holder_set_attribute (holder, GDA_ATTRIBUTE_DESCRIPTION, value);
 			break;
 		case PROP_GDA_TYPE:
-			holder->priv->g_type = g_value_get_ulong (value);
-			break;
-		case PROP_PLUGIN:
-			ptr = g_value_get_string (value);
-			if (holder->priv->plugin) {
-				g_free (holder->priv->plugin);
-				holder->priv->plugin = NULL;
-			}
-			if (ptr)
-				holder->priv->plugin = g_strdup (ptr);
+			if (holder->priv->g_type == GDA_TYPE_NULL)
+				holder->priv->g_type = g_value_get_gtype (value);
+			else
+				g_warning (_("The 'g-type' property cannot be changed"));
 			break;
 		case PROP_NOT_NULL: {
 			gboolean not_null = g_value_get_boolean (value);
@@ -586,6 +592,7 @@ gda_holder_get_property (GObject *object,
 			 GParamSpec *pspec)
 {
 	GdaHolder *holder;
+	const GValue *cvalue;
 
 	holder = GDA_HOLDER (object);
 	if (holder->priv) {
@@ -594,19 +601,21 @@ gda_holder_get_property (GObject *object,
 			g_value_set_string (value, holder->priv->id);
 			break;
 		case PROP_NAME:
-			if (holder->priv->name)
-				g_value_set_string (value, holder->priv->name);
+			cvalue = gda_holder_get_attribute (holder, GDA_ATTRIBUTE_NAME);
+			if (cvalue)
+				g_value_set_string (value, g_value_get_string (cvalue));
 			else
 				g_value_set_string (value, holder->priv->id);
 			break;
 		case PROP_DESCR:
-			g_value_set_string (value, holder->priv->descr);
+			cvalue = gda_holder_get_attribute (holder, GDA_ATTRIBUTE_DESCRIPTION);
+			if (cvalue)
+				g_value_set_string (value, g_value_get_string (cvalue));
+			else
+				g_value_set_string (value, NULL);
 			break;
 		case PROP_GDA_TYPE:
-			g_value_set_ulong (value, holder->priv->g_type);
-			break;
-		case PROP_PLUGIN:
-			g_value_set_string (value, holder->priv->plugin);
+			g_value_set_gtype (value, holder->priv->g_type);
 			break;
 		case PROP_NOT_NULL:
 			g_value_set_boolean (value, gda_holder_get_not_null (holder));
@@ -792,23 +801,23 @@ gda_holder_set_value_str (GdaHolder *holder, GdaDataHandler *dh, const gchar *va
 
 	if (!value || !g_ascii_strcasecmp (value, "NULL")) 
                 return gda_holder_set_value (holder, NULL, error);
-        else {
-                GValue *gdaval = NULL;
+    else {
+		GValue *gdaval = NULL;
 
 		if (!dh)
 			dh = gda_get_default_handler (holder->priv->g_type);
-                if (dh)
-                        gdaval = gda_data_handler_get_value_from_str (dh, value, holder->priv->g_type);
+		if (dh)
+        	gdaval = gda_data_handler_get_value_from_str (dh, value, holder->priv->g_type);
 
-                if (gdaval)
+		if (gdaval)
 			return real_gda_holder_set_value (holder, gdaval, FALSE, error);
-                else {
+        else {
 			g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_STRING_CONVERSION_ERROR,
 				     _("Unable to convert string to '%s' type"), 
 				     gda_g_type_to_string (holder->priv->g_type));
-                        return FALSE;
+        	return FALSE;
 		}
-        }
+	}
 }
 
 /**
@@ -833,6 +842,10 @@ gda_holder_set_value_str (GdaHolder *holder, GdaDataHandler *dh, const gchar *va
  * of which can prevent the change from happening) which can be connected to to have a greater control
  * of which values @holder can have, or implement some business rules.
  *
+ * Note3: if user previously set this holder with gda_holder_take_static_value () the GValue
+ * stored internally will be forgiven and replaced by the @value. User should then
+ * take care of the 'old' static GValue.
+ *
  * Returns: TRUE if value has been set
  */
 gboolean
@@ -858,6 +871,14 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 	gboolean was_valid = gda_holder_is_valid (holder);
 #endif
 
+	/* if the value has been set with gda_holder_take_static_value () you'll be able
+	 * to change the value only with another call to real_gda_holder_set_value 
+	 */
+	if (!holder->priv->is_freeable) {
+		g_warning (_("Can't use this method to set value because there is already a static value"));
+		return FALSE;
+	}
+		
 	/* holder will be changed? */
 	newnull = !value || gda_value_is_null (value);
 	current_val = gda_holder_get_value (holder);
@@ -901,6 +922,7 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 		if (!do_copy && value)
 			gda_value_free (value);
 		holder->priv->invalid_forced = FALSE;
+		holder->priv->valid = newvalid;
 		return TRUE;
 	}
 
@@ -918,6 +940,8 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 	/* new valid status */
 	holder->priv->invalid_forced = FALSE;
 	holder->priv->valid = newvalid;
+	/* we're setting a non-static value, so be sure to flag is as freeable */
+	holder->priv->is_freeable = TRUE;
 
 	/* check is the new value is the default one */
 	holder->priv->default_forced = FALSE;
@@ -928,6 +952,10 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 			 value && (G_VALUE_TYPE (value) == holder->priv->g_type))
 			holder->priv->default_forced = !gda_value_compare (holder->priv->default_value, value);
 	}
+	GValue *att_value;
+	g_value_set_boolean ((att_value = gda_value_new (G_TYPE_BOOLEAN)), holder->priv->default_forced);
+	gda_holder_set_attribute (holder, GDA_ATTRIBUTE_IS_DEFAULT, att_value);
+	gda_value_free (att_value);
 
 	/* real setting of the value */
 	if (holder->priv->full_bind) {
@@ -960,6 +988,170 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 	return newvalid;
 }
 
+static GValue *
+real_gda_holder_set_const_value (GdaHolder *holder, const GValue *value, 
+								 gboolean *value_changed, GError **error)
+{
+	gboolean changed = TRUE;
+	gboolean newvalid;
+	const GValue *current_val;
+	GValue *value_to_return = NULL;
+	gboolean newnull;
+#define DEBUG_HOLDER
+#undef DEBUG_HOLDER
+
+#ifdef DEBUG_HOLDER
+	gboolean was_valid = gda_holder_is_valid (holder);
+#endif
+
+	/* holder will be changed? */
+	newnull = !value || gda_value_is_null (value);
+	current_val = gda_holder_get_value (holder);
+	if (current_val == value)
+		changed = FALSE;
+	else if ((!current_val || gda_value_is_null (current_val)) && newnull) 
+		changed = FALSE;
+	else if (value && current_val &&
+		 (G_VALUE_TYPE (value) == G_VALUE_TYPE (current_val))) 
+		changed = gda_value_differ (value, current_val);
+		
+	/* holder's validity */
+	newvalid = TRUE;
+	if (newnull && holder->priv->not_null) {
+		g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_VALUE_NULL_ERROR,
+			     _("Holder does not allow NULL values"));
+		newvalid = FALSE;
+		changed = TRUE;
+	}
+	else if (!newnull && (G_VALUE_TYPE (value) != holder->priv->g_type)) {
+		g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_VALUE_TYPE_ERROR,
+			     _("Wrong value type: expected type '%s' when value's type is '%s'"),
+			     gda_g_type_to_string (holder->priv->g_type),
+			     gda_g_type_to_string (G_VALUE_TYPE (value)));
+		newvalid = FALSE;
+		changed = TRUE;
+	}
+
+#ifdef DEBUG_HOLDER
+	g_print ("Changed holder %p (%s): value %s --> %s \t(type %d -> %d) VALID: %d->%d CHANGED: %d\n", 
+		 holder, holder->priv->id,
+		 current_val ? gda_value_stringify ((GValue *)current_val) : "_NULL_",
+		 value ? gda_value_stringify ((value)) : "_NULL_",
+		 current_val ? G_VALUE_TYPE ((GValue *)current_val) : 0,
+		 value ? G_VALUE_TYPE (value) : 0, 
+		 was_valid, newvalid, changed);
+#endif
+	
+
+	/* end of procedure if the value has not been changed, after calculating the holder's validity */
+	if (!changed) {
+		holder->priv->invalid_forced = FALSE;
+		holder->priv->valid = newvalid;
+#ifdef DEBUG_HOLDER		
+		g_print ("Holder is not changed, returning %p\n", holder->priv->value);
+#endif		
+
+		/* set the changed status */
+		*value_changed = FALSE;
+		return holder->priv->value;
+	}
+	else {
+		*value_changed = TRUE;
+	}
+
+	/* check if we are allowed to change value */
+	GError *lerror = NULL;
+	g_signal_emit (holder, gda_holder_signals[VALIDATE_CHANGE], 0, value, &lerror);
+	if (lerror) {
+		/* change refused by signal callback */
+		g_propagate_error (error, lerror);
+		return NULL;
+	}
+
+	/* new valid status */
+	holder->priv->invalid_forced = FALSE;
+	holder->priv->valid = newvalid;
+	/* we're setting a static value, so be sure to flag is as unfreeable */
+	holder->priv->is_freeable = FALSE;
+
+	/* check is the new value is the default one */
+	holder->priv->default_forced = FALSE;
+	if (holder->priv->default_value) {
+		if ((G_VALUE_TYPE (holder->priv->default_value) == GDA_TYPE_NULL) && newnull)
+			holder->priv->default_forced = TRUE;
+		else if ((G_VALUE_TYPE (holder->priv->default_value) == holder->priv->g_type) &&
+			 value && (G_VALUE_TYPE (value) == holder->priv->g_type))
+			holder->priv->default_forced = !gda_value_compare (holder->priv->default_value, value);
+	}
+	GValue *att_value;
+	g_value_set_boolean ((att_value = gda_value_new (G_TYPE_BOOLEAN)), holder->priv->default_forced);
+	gda_holder_set_attribute (holder, GDA_ATTRIBUTE_IS_DEFAULT, att_value);
+	gda_value_free (att_value);
+
+	/* real setting of the value */
+	if (holder->priv->full_bind) {
+#ifdef DEBUG_HOLDER
+		g_print ("Holder %p is alias of holder %p => propagating changes to holder %p\n",
+			 holder, holder->priv->full_bind, holder->priv->full_bind);
+#endif
+		return real_gda_holder_set_const_value (holder->priv->full_bind, value, 
+												value_changed, error);
+	}
+	else {
+		if (holder->priv->value) {
+			value_to_return = holder->priv->value;
+			holder->priv->value = NULL;
+		}
+
+		if (value) {
+			if (newvalid) {
+				holder->priv->value = (GValue*)value;
+			}
+		}
+
+		g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
+	}
+
+	return value_to_return;
+}
+
+/**
+ * gda_holder_take_static_value
+ * @holder: a #GdaHolder object
+ * @value: a const value to set the holder to
+ * @value_changed: a boolean set with TRUE if the value changes, FALSE elsewhere.
+ * @error: a place to store errors, or %NULL
+ *
+ * Sets the const value within the holder. If @holder is an alias for another
+ * holder, then the value is also set for that other holder.
+ *
+ * The value will not be freed, and user should take care of it, either for its
+ * freeing or for its correct value at the moment of query.
+ * 
+ * If the value is not different from the one already contained within @holder,
+ * then @holder is not chaged and no signal is emitted.
+ *
+ * Note1: if @holder can't accept the @value value, then this method returns NULL, and @holder will be left
+ * in an invalid state.
+ *
+ * Note2: before the change is accepted by @holder, the "validate-change" signal will be emitted (the value
+ * of which can prevent the change from happening) which can be connected to to have a greater control
+ * of which values @holder can have, or implement some business rules.
+ *
+ * Returns: NULL if an error occurred or if the previous GValue was NULL itself. It returns
+ * the static GValue user set previously, so that he can free it.
+ *
+ */
+GValue *
+gda_holder_take_static_value (GdaHolder *holder, const GValue *value, gboolean *value_changed,
+							  GError **error)
+{
+	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
+	g_return_val_if_fail (holder->priv, FALSE);
+
+	return real_gda_holder_set_const_value (holder, value, value_changed, error);
+}
+
 /**
  * gda_holder_force_invalid
  * @holder: a #GdaHolder object
@@ -986,7 +1178,8 @@ gda_holder_force_invalid (GdaHolder *holder)
 	holder->priv->valid = FALSE;
 	
 	if (holder->priv->value) {
-		gda_value_free (holder->priv->value);
+		if (holder->priv->is_freeable)
+			gda_value_free (holder->priv->value);
 		holder->priv->value = NULL;
 	}
 
@@ -1048,12 +1241,18 @@ gda_holder_set_value_to_default (GdaHolder *holder)
 		holder->priv->default_forced = TRUE;
 		holder->priv->invalid_forced = FALSE;
 		if (holder->priv->value) {
-			gda_value_free (holder->priv->value);
+			if (holder->priv->is_freeable)
+				gda_value_free (holder->priv->value);
 			holder->priv->value = NULL;
 		}
 	}
 
+	GValue *att_value;
+	g_value_set_boolean ((att_value = gda_value_new (G_TYPE_BOOLEAN)), TRUE);
+	gda_holder_set_attribute (holder, GDA_ATTRIBUTE_IS_DEFAULT, att_value);
+	gda_value_free (att_value);
 	g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
+
 	return TRUE;
 }
 
@@ -1138,6 +1337,11 @@ gda_holder_set_default_value (GdaHolder *holder, const GValue *value)
 		holder->priv->default_value = gda_value_copy ((GValue *)value);
 	}
 	
+	GValue *att_value;
+	g_value_set_boolean ((att_value = gda_value_new (G_TYPE_BOOLEAN)), holder->priv->default_forced);
+	gda_holder_set_attribute (holder, GDA_ATTRIBUTE_IS_DEFAULT, att_value);
+	gda_value_free (att_value);
+
 	/* don't emit the "changed" signal */
 }
 
@@ -1361,7 +1565,8 @@ gda_holder_set_full_bind (GdaHolder *holder, GdaHolder *alias_of)
 
 		/* get rid of the internal holder's value */
 		if (holder->priv->value) {
-			gda_value_free (holder->priv->value);
+			if (holder->priv->is_freeable)
+				gda_value_free (holder->priv->value);
 			holder->priv->value = NULL;
 		}
 
@@ -1438,3 +1643,48 @@ gda_holder_get_alphanum_id (GdaHolder *holder)
 	return gda_text_to_alphanum (holder->priv->id);
 }
 
+/**
+ * gda_holder_get_attribute
+ * @holder: a #GdaHolder
+ * @attribute: attribute name as a string
+ *
+ * Get the value associated to a named attribute.
+ *
+ * Attributes can have any name, but Libgda proposes some default names, see <link linkend="libgda-40-Attributes-manager.synopsis">this section</link>.
+ *
+ * Returns: a read-only #GValue, or %NULL if not attribute named @attribute has been set for @holder
+ */
+const GValue *
+gda_holder_get_attribute (GdaHolder *holder, const gchar *attribute)
+{
+	g_return_val_if_fail (GDA_IS_HOLDER (holder), NULL);
+	return gda_attributes_manager_get (gda_holder_attributes_manager, holder, attribute);
+}
+
+/**
+ * gda_holder_set_attribute
+ * @holder: a #GdaHolder
+ * @attribute: attribute name as a static string
+ * @value: a #GValue, or %NULL
+ *
+ * Set the value associated to a named attribute.
+ *
+ * Attributes can have any name, but Libgda proposes some default names, see <link linkend="libgda-40-Attributes-manager.synopsis">this section</link>.
+ * If there is already an attribute named @attribute set, then its value is replaced with the new @value, 
+ * except if @value is %NULL, in which case the attribute is removed.
+ *
+ * Warning: @sttribute should be a static string (no copy of it is made), so the string should exist as long as the @holder
+ * object exists.
+ */
+void
+gda_holder_set_attribute (GdaHolder *holder, const gchar *attribute, const GValue *value)
+{
+	const GValue *cvalue;
+	g_return_if_fail (GDA_IS_HOLDER (holder));
+
+	cvalue = gda_attributes_manager_get (gda_holder_attributes_manager, holder, attribute);
+	if (cvalue && !gda_value_differ (cvalue, value))
+		return;
+
+	gda_attributes_manager_set (gda_holder_attributes_manager, holder, attribute, value);
+}
