@@ -57,6 +57,7 @@ struct _GdaHolderClass
 	void                     (*changed)          (GdaHolder *holder);
 	void                     (*source_changed)   (GdaHolder *holder);
 	GError                  *(*validate_change)  (GdaHolder *holder, const GValue *new_value);
+	void                     (*att_changed)      (GdaHolder *holder, const gchar *att_name, const GValue *att_value);
 };
 
 GType               gda_holder_get_type                (void) G_GNUC_CONST;
@@ -75,6 +76,7 @@ const GValue       *gda_holder_get_value               (GdaHolder *holder);
 gchar              *gda_holder_get_value_str           (GdaHolder *holder, GdaDataHandler *dh);
 gboolean            gda_holder_set_value               (GdaHolder *holder, const GValue *value, GError **error);
 gboolean            gda_holder_take_value              (GdaHolder *holder, GValue *value, GError **error);
+GValue             *gda_holder_take_static_value       (GdaHolder *holder, const GValue *value, gboolean *value_changed, GError **error);
 gboolean            gda_holder_set_value_str           (GdaHolder *holder, GdaDataHandler *dh, const gchar *value, GError **error);
 
 const GValue       *gda_holder_get_default_value       (GdaHolder *holder);
@@ -96,6 +98,9 @@ GdaDataModel       *gda_holder_get_source_model         (GdaHolder *holder, gint
 gboolean            gda_holder_set_bind                 (GdaHolder *holder, GdaHolder *bind_to, GError **error);
 GdaHolder          *gda_holder_get_bind                 (GdaHolder *holder);
 
+const GValue       *gda_holder_get_attribute            (GdaHolder *holder, const gchar *attribute);
+void                gda_holder_set_attribute            (GdaHolder *holder, const gchar *attribute, const GValue *value);
+
 G_END_DECLS
 
 #endif
