@@ -19,11 +19,11 @@
 
 #include <stdarg.h>
 #include <stdio.h>
-#ifndef LIBGDA_WIN32
+#include <glib.h>
+#ifndef G_OS_WIN32
 #include <syslog.h>
 #endif
 #include <time.h>
-#include <glib.h>
 #include <glib/gmem.h>
 #include <glib/gmessages.h>
 #include <glib/gstrfuncs.h>
@@ -31,6 +31,9 @@
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-log.h>
 
+static GStaticRecMutex gda_mutex = G_STATIC_REC_MUTEX_INIT;
+#define GDA_LOG_LOCK() g_static_rec_mutex_lock(&gda_mutex)
+#define GDA_LOG_UNLOCK() g_static_rec_mutex_unlock(&gda_mutex)
 static gboolean log_enabled = TRUE;
 static gboolean log_opened = FALSE;
 
@@ -46,13 +49,15 @@ static gboolean log_opened = FALSE;
 void
 gda_log_enable (void)
 {
+	GDA_LOG_LOCK ();
 	log_enabled = TRUE;
 	if (!log_opened) {
-#ifndef LIBGDA_WIN32	
+#ifndef G_OS_WIN32	
 		openlog (g_get_prgname (), LOG_CONS | LOG_NOWAIT | LOG_PID, LOG_USER);
 #endif		
 		log_opened = TRUE;
 	}
+	GDA_LOG_UNLOCK ();
 }
 
 /**
@@ -63,13 +68,15 @@ gda_log_enable (void)
 void
 gda_log_disable (void)
 {
+	GDA_LOG_LOCK ();
 	log_enabled = FALSE;
 	if (log_opened) {
-#ifndef LIBGDA_WIN32		
+#ifndef G_OS_WIN32		
 		closelog ();
 #endif
 		log_opened = FALSE;
 	}
+	GDA_LOG_UNLOCK ();
 }
 
 /**
@@ -101,6 +108,7 @@ gda_log_message (const gchar *format, ...)
 	if (!log_enabled)
 		return;
 
+	GDA_LOG_LOCK ();
 	if (!log_opened)
 		gda_log_enable ();
 
@@ -108,12 +116,13 @@ gda_log_message (const gchar *format, ...)
 	va_start (args, format);
 	msg = g_strdup_vprintf (format, args);
 	va_end (args);
-#ifdef LIBGDA_WIN32
+#ifdef G_OS_WIN32
 	g_log ("Gda", G_LOG_LEVEL_INFO, "%s", msg);
 #else
 	syslog (LOG_USER | LOG_INFO, "%s", msg);
 #endif
 	g_free (msg);
+	GDA_LOG_UNLOCK ();
 }
 
 /**
@@ -134,6 +143,7 @@ gda_log_error (const gchar * format, ...)
 	if (!log_enabled)
 		return;
 
+	GDA_LOG_LOCK ();
 	if (!log_opened)
 		gda_log_enable ();
 
@@ -141,10 +151,11 @@ gda_log_error (const gchar * format, ...)
 	va_start (args, format);
 	msg = g_strdup_vprintf (format, args);
 	va_end (args);
-#ifdef LIBGDA_WIN32
-	g_log ("Gda", G_LOG_LEVEL_ERROR, "%s", msg);
+#ifdef G_OS_WIN32
+	g_log ("Gda", G_LOG_LEVEL_DEBUG, "%s", msg);
 #else
 	syslog (LOG_USER | LOG_ERR, "%s", msg);
 #endif
  	g_free (msg);
+	GDA_LOG_UNLOCK ();
 }
