@@ -38,6 +38,8 @@
 #endif
 #include <glib/gi18n-lib.h>
 
+extern gchar *gda_lang_locale;
+
 #define CLASS(operation) (GDA_SERVER_OPERATION_CLASS (G_OBJECT_GET_CLASS (operation)))
 
 static void gda_server_operation_class_init (GdaServerOperationClass *klass);
@@ -137,7 +139,7 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 
 	/* signals */
 	gda_server_operation_signals[SEQUENCE_ITEM_ADDED] =
-		g_signal_new ("sequence_item_added",
+		g_signal_new ("sequence-item-added",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaServerOperationClass, seq_item_added),
@@ -145,7 +147,7 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 			      gda_marshal_VOID__STRING_INT, G_TYPE_NONE,
 			      2, G_TYPE_STRING, G_TYPE_INT);
 	gda_server_operation_signals[SEQUENCE_ITEM_REMOVE] =
-		g_signal_new ("sequence_item_remove",
+		g_signal_new ("sequence-item-remove",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaServerOperationClass, seq_item_remove),
@@ -167,14 +169,14 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 							      GDA_TYPE_CONNECTION,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_PROV,
-					 g_param_spec_object ("provider_obj", NULL, NULL, 
+					 g_param_spec_object ("provider", NULL, NULL, 
 							      GDA_TYPE_SERVER_PROVIDER,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_SPEC_FILE,
-					 g_param_spec_string ("spec_file", NULL, NULL, 
+					 g_param_spec_string ("spec-filename", NULL, NULL, 
 							      NULL, G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_OP_TYPE,
-					 g_param_spec_int ("op_type", NULL, NULL, 
+					 g_param_spec_int ("op-type", NULL, NULL, 
 							   0, GDA_SERVER_OPERATION_LAST - 1, 
 							   0, G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 }
@@ -296,19 +298,14 @@ node_destroy (GdaServerOperation *op, Node *node)
 	case GDA_SERVER_OPERATION_NODE_SEQUENCE: {
 		GSList *list;
 
-		list = node->d.seq.seq_tmpl;
-		while (list) {
+		for (list = node->d.seq.seq_tmpl; list; list = list->next)
 			node_destroy (op, NODE (list->data));
-			list = g_slist_next (list);
-		}
 		g_slist_free (node->d.seq.seq_tmpl);
 
-		list = node->d.seq.seq_items;
-		while (list) {
+		for (list = node->d.seq.seq_items; list; list = list->next)
 			node_destroy (op, NODE (list->data));
-			list = g_slist_next (list);
-		}
 		g_slist_free (node->d.seq.seq_items);
+
 		g_free (node->d.seq.name);
 		g_free (node->d.seq.descr);
 		break;
@@ -316,11 +313,8 @@ node_destroy (GdaServerOperation *op, Node *node)
 	case GDA_SERVER_OPERATION_NODE_SEQUENCE_ITEM: {
 		GSList *list;
 
-		list = node->d.seq_item_nodes;
-		while (list) {
+		for (list = node->d.seq_item_nodes; list; list = list->next)
 			node_destroy (op, NODE (list->data));
-			list = g_slist_next (list);
-		}
 		g_slist_free (node->d.seq_item_nodes);
 		break;
 	}
@@ -330,8 +324,10 @@ node_destroy (GdaServerOperation *op, Node *node)
 	}
 
 	g_free (node->path_name);
-	op->priv->topnodes = g_slist_remove (op->priv->topnodes, node);
-	op->priv->allnodes = g_slist_remove (op->priv->allnodes, node);
+	if (op) {
+		op->priv->topnodes = g_slist_remove (op->priv->topnodes, node);
+		op->priv->allnodes = g_slist_remove (op->priv->allnodes, node);
+	}
 
 	g_free (node);
 }
@@ -348,14 +344,15 @@ node_find (GdaServerOperation *op, const gchar *path)
 	if (!path || !*path || (*path != '/'))
 		return NULL;
 
-	list = op->priv->allnodes;
-	while (list && !node) {
+	for (list = op->priv->allnodes; list; list = list->next) {
 		gchar *str;
 		str = node_get_complete_path (op, NODE (list->data));
-		if (!strcmp (str, path))
+		if (!strcmp (str, path)) {
 			node = NODE (list->data);
+			g_free (str);
+			break;
+		}
 		g_free (str);
-		list = list->next;
 	}
 	/*g_print ("%s(%s) => %p\n", __FUNCTION__, path, node);*/
 	return node;
@@ -438,8 +435,7 @@ node_get_complete_path (GdaServerOperation *op, Node *node)
 		return NULL;
 
 	string = g_string_new ("");
-	lnode = node;
-	while (lnode) {
+	for (lnode = node; lnode; lnode = lnode->parent) {
 		if (lnode->type == GDA_SERVER_OPERATION_NODE_SEQUENCE_ITEM) {
 			gchar *str;
 
@@ -452,7 +448,6 @@ node_get_complete_path (GdaServerOperation *op, Node *node)
 		else 
 			g_string_prepend (string, lnode->path_name);
 		g_string_prepend_c (string, '/');
-		lnode = lnode->parent;
 	}
 
 	retval = string->str;
@@ -462,7 +457,7 @@ node_get_complete_path (GdaServerOperation *op, Node *node)
 	return retval;
 }
 
-static GSList *load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root);
+static GSList *load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root, GError **error);
 
 /* add a new item to @node and inserts it into @op's private structures */
 static void
@@ -470,6 +465,7 @@ sequence_add_item (GdaServerOperation *op, Node *node)
 {
 	gchar *path, *seq_path;
 	Node *new_node;
+	GSList *seq_item_nodes, *list;
 
 	g_assert (node);
 	g_assert (node->type == GDA_SERVER_OPERATION_NODE_SEQUENCE);
@@ -477,13 +473,18 @@ sequence_add_item (GdaServerOperation *op, Node *node)
 	seq_path = node_get_complete_path (op, node);
 	path = g_strdup_printf ("%s/%d", seq_path, g_slist_length (node->d.seq.seq_items));
 
+	seq_item_nodes = load_xml_spec (op, node->d.seq.xml_spec, path, NULL);
+	g_assert (seq_item_nodes);
+
 	new_node = node_new (node, GDA_SERVER_OPERATION_NODE_SEQUENCE_ITEM, NULL);
 	op->priv->allnodes = g_slist_append (op->priv->allnodes, new_node);
 	new_node->d.seq_item_nodes = NULL;
 	new_node->status = node->status;
 	node->d.seq.seq_items = g_slist_append (node->d.seq.seq_items, new_node);
 
-	new_node->d.seq_item_nodes = load_xml_spec (op, node->d.seq.xml_spec, path);
+	new_node->d.seq_item_nodes = seq_item_nodes;
+	for (list = seq_item_nodes; list; list = list->next)
+		((Node*) list->data)->parent = new_node;
 
 	clean_nodes_info_cache (op);
 #ifdef GDA_DEBUG_signal
@@ -522,10 +523,10 @@ gda_server_operation_set_property (GObject *object,
 			if (op->priv->cnc) {
 				g_object_ref (op->priv->cnc);
 
-				if (gda_connection_get_provider_obj (op->priv->cnc)) {
+				if (gda_connection_get_provider (op->priv->cnc)) {
 					if (op->priv->prov)
 						g_object_unref (op->priv->prov);
-					op->priv->prov = gda_connection_get_provider_obj (op->priv->cnc);
+					op->priv->prov = gda_connection_get_provider (op->priv->cnc);
 					g_object_ref (op->priv->prov);
 					op->priv->prov_set = TRUE;
 				}
@@ -630,9 +631,17 @@ gda_server_operation_set_property (GObject *object,
 		}
 	}
 
-	if (op->priv->xml_spec_doc && op->priv->cnc_set && op->priv->prov_set) 
+	if (!op->priv->topnodes && op->priv->xml_spec_doc && op->priv->cnc_set && op->priv->prov_set) {
 		/* load XML file */
-		op->priv->topnodes = load_xml_spec (op, xmlDocGetRootElement (op->priv->xml_spec_doc), NULL);
+		GError *lerror = NULL;
+		op->priv->topnodes = load_xml_spec (op, xmlDocGetRootElement (op->priv->xml_spec_doc), NULL, &lerror);
+		if (!op->priv->topnodes) {
+			g_warning (_("Could not load XML specifications: %s"),
+				   lerror && lerror->message ? lerror->message : _("No detail"));
+			if (lerror)
+				g_error_free (lerror);
+		}
+	}
 }
 
 static void
@@ -686,22 +695,19 @@ xml_validity_error_func (void *ctx, const char *msg, ...)
         g_object_set_data (G_OBJECT (op), "xmlerror", newerr);
 }
 
+/*
+ * Warning: the new nodes' parent is not set!
+ */
 static GSList *
-load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
+load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root, GError **error)
 {
 	xmlNodePtr node;
-	const gchar *lang;
+	const gchar *lang = gda_lang_locale;
 	GSList *retlist = NULL;
 	Node *parent = NULL;
 
 	if (root) 
 		parent = node_find (op, root);
-
-#ifdef HAVE_LC_MESSAGES
-        lang = setlocale (LC_MESSAGES, NULL);
-#else
-        lang = setlocale (LC_CTYPE, NULL);
-#endif
 	
 	g_assert (specnode);
 
@@ -712,7 +718,7 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 		node = specnode->children;
 		while (node && (xmlNodeIsText (node) || strcmp ((gchar*)node->name, "sources"))) 
 		node = node->next; 
-		if (node && !strcmp ((gchar*)node->name, "sources")){
+		if (node && !strcmp ((gchar*)node->name, "sources")) {
 			for (node = node->xmlChildrenNode; (node != NULL); node = node->next) {
 				if (xmlNodeIsText (node)) 
 					continue;
@@ -725,7 +731,11 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 					errors = gda_data_model_import_get_errors (GDA_DATA_MODEL_IMPORT (model));
 					if (errors) {
 						g_object_unref (model);
-						model = NULL;
+						if (sources) {
+							g_slist_foreach (sources, (GFunc) g_object_unref, NULL);
+							g_slist_free (sources);
+							return NULL;
+						}
 					}
 					else  {
 						xmlChar *str;
@@ -745,7 +755,6 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 	while (node) {
 		xmlChar *id, *this_lang;
 		gchar *complete_path = NULL, *path_name = NULL;
-		GError *error = NULL;
 		Node *opnode = NULL;
 		Node *old_opnode;
 
@@ -794,13 +803,8 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 		if (!strcmp ((gchar*)node->name, "parameters")) {
 			GdaSet *plist;
 
-			plist = gda_set_new_from_spec_node (node, &error);
-			if (!plist) {
-				g_warning (_("GdaServerOperation: error loading GdaSet spec %s: '%s'"), 
-					   complete_path, error && error->message ? error->message : _("No detail"));
-				g_error_free (error);
-			}
-			else {
+			plist = gda_set_new_from_spec_node (node, error);
+			if (plist) {
 				opnode = node_new (parent, GDA_SERVER_OPERATION_NODE_PARAMLIST, path_name);
 				opnode->d.plist = plist;
 			}
@@ -810,10 +814,7 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 			GdaDataModel *import;
 
 			import = gda_data_model_import_new_xml_node (node);
-			if (!import) {
-				TO_IMPLEMENT;
-			}
-			else {
+			if (import) {
 				GdaDataModel *model;
 				model = (GdaDataModel*) gda_data_model_array_copy_model (import, NULL);
 				opnode = node_new (parent, GDA_SERVER_OPERATION_NODE_DATA_MODEL, path_name);
@@ -859,8 +860,13 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 				xmlFree (prop);
 			}
 
-			seq_tmpl = load_xml_spec (op, node, complete_path);
-			opnode->d.seq.seq_tmpl = seq_tmpl;
+			seq_tmpl = load_xml_spec (op, node, complete_path, error);
+			if (! seq_tmpl) {
+				node_destroy (NULL, opnode);
+				opnode = NULL;
+			}
+			else
+				opnode->d.seq.seq_tmpl = seq_tmpl;
 		}
 
 		/* GDA_SERVER_OPERATION_NODE_PARAM */
@@ -871,16 +877,17 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 			/* find data type and create GdaHolder */
 			gdatype = xmlGetProp (node, BAD_CAST "gdatype");
 			param = GDA_HOLDER (g_object_new (GDA_TYPE_HOLDER,
-							  "g_type", 
+							  "g-type", 
 							  gdatype ? gda_g_type_from_string ((gchar*) gdatype) : G_TYPE_STRING,
 							  NULL));
-			if (gdatype) xmlFree (gdatype);
+			if (gdatype)
+				xmlFree (gdatype);
 			
 			/* set parameter's attributes */
-			gda_utility_holder_load_attributes (param, node, op->priv->sources);
-			
-			opnode = node_new (parent, GDA_SERVER_OPERATION_NODE_PARAM, path_name);
-			opnode->d.param = param;
+			if (gda_utility_holder_load_attributes (param, node, op->priv->sources, error)) {
+				opnode = node_new (parent, GDA_SERVER_OPERATION_NODE_PARAM, path_name);
+				opnode->d.param = param;
+			}
 		}
 		else {
 			node = node->next;
@@ -914,7 +921,12 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 				}
 			}
 		}
-		
+		else {
+			g_free (path_name);
+			g_free (complete_path);
+			g_slist_free (retlist);
+			return NULL;
+		}
 		g_free (path_name);
 		g_free (complete_path);
 		node = node->next;
@@ -942,7 +954,7 @@ gda_server_operation_new (GdaServerOperationType op_type, const gchar *xml_file)
 {
 	GObject *obj;
 
-	obj = g_object_new (GDA_TYPE_SERVER_OPERATION, "op_type", op_type, "spec_file", xml_file, NULL);
+	obj = g_object_new (GDA_TYPE_SERVER_OPERATION, "op-type", op_type, "spec-filename", xml_file, NULL);
 #ifdef GDA_DEBUG_NO
 	{
 		g_print ("New GdaServerOperation:\n");
@@ -1133,7 +1145,7 @@ gda_server_operation_op_type_to_string (GdaServerOperationType type)
 	}
 }
 
-static void node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent);
+static gboolean node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent);
 
 /**
  * gda_server_operation_save_data_to_xml
@@ -1159,18 +1171,21 @@ gda_server_operation_save_data_to_xml (GdaServerOperation *op, GError **error)
 	xmlSetProp (topnode, BAD_CAST "type", 
 		    BAD_CAST gda_server_operation_op_type_to_string (gda_server_operation_get_op_type (op)));
 
-	list = op->priv->topnodes;
-	while (list) {
-		node_save (op, NODE (list->data), topnode);
-		list = g_slist_next (list);
+	for (list = op->priv->topnodes; list; list = list->next) {
+		if (!node_save (op, NODE (list->data), topnode)) {
+			xmlFreeNode (topnode);
+			topnode = NULL;
+			break;
+		}
 	}
 
 	return topnode;
 }
 
-static void
+static gboolean
 node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 {
+	gboolean retval = TRUE;
 	xmlNodePtr node;
 	GSList *list;
 	gchar *complete_path;
@@ -1179,8 +1194,7 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 	complete_path = node_get_complete_path (op, opnode);
 	switch (opnode->type) {
 	case GDA_SERVER_OPERATION_NODE_PARAMLIST:
-		list = opnode->d.plist->holders;
-		while (list) {
+		for (list = opnode->d.plist->holders; list; list = list->next) {
 			gchar *path;
 			const GValue *value;
 			gchar *str;
@@ -1194,20 +1208,19 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 				else
 					str = gda_value_stringify (value);
 			}
-			node = xmlNewChild (parent, NULL, BAD_CAST "op_data", (xmlChar*)str);
+			node = xmlNewTextChild (parent, NULL, BAD_CAST "op_data", (xmlChar*)str);
 			g_free (str);
 
 			path = g_strdup_printf ("%s/%s", complete_path, gda_holder_get_id (GDA_HOLDER (list->data)));
 			xmlSetProp(node, (xmlChar*)"path", (xmlChar*)path);
 			g_free (path);
-
-			list = g_slist_next (list);
 		}
 		break;
 	case GDA_SERVER_OPERATION_NODE_DATA_MODEL:
 		node = xmlNewChild (parent, NULL, BAD_CAST "op_data", NULL);
 		xmlSetProp(node, (xmlChar*)"path", (xmlChar*)complete_path);
-		gda_utility_data_model_dump_data_to_xml (opnode->d.model, node, NULL, 0, NULL, 0, TRUE);
+		if (!gda_utility_data_model_dump_data_to_xml (opnode->d.model, node, NULL, 0, NULL, 0, TRUE))
+			retval = FALSE;
 		break;
 	case GDA_SERVER_OPERATION_NODE_PARAM: {
 		const GValue *value;
@@ -1222,7 +1235,7 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 			else
 				str = gda_value_stringify (value);
 		}
-		node = xmlNewChild (parent, NULL, BAD_CAST "op_data", (xmlChar*)str);
+		node = xmlNewTextChild (parent, NULL, BAD_CAST "op_data", (xmlChar*)str);
 		g_free (str);
 		xmlSetProp(node, (xmlChar*)"path", (xmlChar*)complete_path);
 		break;
@@ -1231,14 +1244,20 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 		GSList *list;
 		
 		for (list =  opnode->d.seq.seq_items; list; list = list->next) 
-			node_save (op, NODE (list->data), parent);
+			if (!node_save (op, NODE (list->data), parent)) {
+				retval = FALSE;
+				break;
+			}
 		break;
 	}
 	case GDA_SERVER_OPERATION_NODE_SEQUENCE_ITEM: {
 		GSList *list;
 		
 		for (list =  opnode->d.seq_item_nodes; list; list = list->next) 
-			node_save (op, NODE (list->data), parent);
+			if (! node_save (op, NODE (list->data), parent)) {
+				retval = FALSE;
+				break;
+			}
 		break;
 	}
 	default:
@@ -1246,6 +1265,7 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 	}
 
 	g_free (complete_path);
+	return retval;
 }
 
 /**
@@ -1359,12 +1379,8 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 								GValue *v;
 								v = gda_value_new_from_string ((gchar*)contents->content, 
 											       gda_holder_get_g_type (param));
-								if (!gda_holder_take_value (param, v)) {
-									g_set_error (error, 0, 0,
-										     _("Could not set parameter '%s' to value '%s'"), 
-										     prop, cur->content);
+								if (!gda_holder_take_value (param, v, error)) 
 									allok = FALSE;
-								}
 							}
 						}
 					}
@@ -1384,11 +1400,8 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 						GValue *v;
 						v = gda_value_new_from_string ((gchar*)contents->content, 
 									       gda_holder_get_g_type (opnode->d.param));
-						if (!gda_holder_take_value (opnode->d.param, v)) {
-							g_set_error (error, 0, 0,
-								     _("Could not set parameter '%s' to value '%s'"), prop, cur->content);
+						if (!gda_holder_take_value (opnode->d.param, v, error)) 
 							allok = FALSE;
-						}
 					}
 					break;
 				}
@@ -1437,12 +1450,9 @@ gda_server_operation_get_root_nodes (GdaServerOperation *op)
 	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), NULL);
 	g_return_val_if_fail (op->priv, NULL);	
 
-	list = op->priv->topnodes;
-	retval = g_new0 (gchar *, g_slist_length (list) + 1);
-	while (list) {
+	retval = g_new0 (gchar *, g_slist_length (op->priv->topnodes) + 1);
+	for (list = op->priv->topnodes; list; list = list->next)
 		retval [i++] = node_get_complete_path (op, NODE (list->data));
-		list = g_slist_next (list);
-	}
 
 	return retval;
 }
@@ -1589,11 +1599,8 @@ gda_server_operation_get_sequence_item_names (GdaServerOperation *op, const gcha
 		list = node->d.seq_item_nodes;
 	i = 0;
 	retval = g_new0 (gchar *, g_slist_length (list) + 1);
-	while (list) {
+	for (; list; list = list->next, i++)
 		retval [i] = node_get_complete_path (op, NODE (list->data));
-		i++;
-		list = g_slist_next (list);
-	}
 
 	return retval;
 }
@@ -1864,7 +1871,7 @@ gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_for
 				if (row >= 0) 
 					value = gda_data_model_get_value_at (node_info->model, 
 									     gda_column_get_position (node_info->column), 
-									     row);
+									     row, NULL);
 				g_free(extension);
 			}
 			g_free (str);
@@ -1984,12 +1991,8 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 									       gda_holder_get_g_type (param));
 					else
 						v = gda_value_new_null ();
-					if (!gda_holder_take_value (param, v)) {
-						g_set_error (error, 0, 0,
-							     _("Could not set parameter '%s' to value '%s'"), 
-							     path, value);
+					if (!gda_holder_take_value (param, v, error))
 						allok = FALSE;
-					}
 				}
 			}
 			break;
@@ -2047,12 +2050,8 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 							       gda_holder_get_g_type (opnode->d.param));
 			else
 				v = gda_value_new_null ();
-			if (!gda_holder_take_value (opnode->d.param, v)) {
-				g_set_error (error, 0, 0,
-					     _("Could not set parameter '%s' to value '%s'"), 
-					     path, value);
+			if (!gda_holder_take_value (opnode->d.param, v, error))
 				allok = FALSE;
-			}
 			break;
 		}
 		case GDA_SERVER_OPERATION_NODE_SEQUENCE:
@@ -2094,8 +2093,7 @@ gda_server_operation_is_valid (GdaServerOperation *op, const gchar *xml_file, GE
 
 	if (!xml_file) {
 		/* basic validity test */
-		list = op->priv->allnodes;
-		while (list && valid) {
+		for (list = op->priv->allnodes; list; list = list->next) {
 			Node *node;
 			
 			node = NODE (list->data);
@@ -2110,22 +2108,16 @@ gda_server_operation_is_valid (GdaServerOperation *op, const gchar *xml_file, GE
 						valid = FALSE;
 						g_set_error (error, 0, 0,
 							     _("Missing required value for '%s'"), path);
+						break;
 					}
 					g_free (path);
 				}
 				else if (node->type == GDA_SERVER_OPERATION_NODE_PARAMLIST) {
-					valid = gda_set_is_valid (node->d.plist);
-					if (!valid) {
-						gchar *path;
-
-						path = node_get_complete_path (op, node);
-						g_set_error (error, 0, 0,
-							     _("Missing required value for list of parameters '%s'"), path);
-						g_free (path);
-					}
+					valid = gda_set_is_valid (node->d.plist, error);
+					if (!valid)
+						break;
 				}
 			}
-			list = list->next;
 		}
 	}
 	else {
