@@ -123,7 +123,9 @@ gda_server_provider_get_data_handler_default (GdaServerProvider *provider, GdaCo
 	    (type == GDA_TYPE_USHORT) ||
 	    (type == G_TYPE_CHAR) ||
 	    (type == G_TYPE_UCHAR) ||
-	    (type == G_TYPE_UINT)) {
+	    (type == G_TYPE_UINT) ||
+	    (type == G_TYPE_LONG) ||
+	    (type == G_TYPE_ULONG)) {
 		dh = gda_server_provider_handler_find (provider, NULL, type, NULL);
 		if (!dh) {
 			dh = gda_handler_numerical_new ();
@@ -138,6 +140,8 @@ gda_server_provider_get_data_handler_default (GdaServerProvider *provider, GdaCo
 			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_CHAR, NULL);
 			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_UCHAR, NULL);
 			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_UINT, NULL);
+			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_ULONG, NULL);
+			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_LONG, NULL);
 			g_object_unref (dh);
 		}
 	}
@@ -168,11 +172,11 @@ gda_server_provider_get_data_handler_default (GdaServerProvider *provider, GdaCo
 			g_object_unref (dh);
 		}
 	}
-	else if (type == G_TYPE_ULONG) {
+	else if (type == G_TYPE_GTYPE) {
 		dh = gda_server_provider_handler_find (provider, NULL, type, NULL);
 		if (!dh) {
 			dh = gda_handler_type_new ();
-			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_ULONG, NULL);
+			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_GTYPE, NULL);
 			g_object_unref (dh);
 		}
 	}
@@ -180,6 +184,75 @@ gda_server_provider_get_data_handler_default (GdaServerProvider *provider, GdaCo
 	return dh;
 }
 
+static gboolean
+param_to_null_foreach (GdaSqlAnyPart *part, gpointer data, GError **error)
+{
+	if (part->type == GDA_SQL_ANY_EXPR) {
+		GdaSqlExpr *expr = (GdaSqlExpr*) part;
+		if (expr->param_spec) {
+			GType type = expr->param_spec->g_type;
+			gda_sql_param_spec_free (expr->param_spec);
+			expr->param_spec = NULL;
+
+			if (!expr->value) {
+				if (type != GDA_TYPE_NULL)
+					expr->value = gda_value_new_from_string ("0", type);
+				else
+					g_value_set_int ((expr->value = gda_value_new (G_TYPE_INT)), 0);
+			}
+		}
+	}
+	return TRUE;
+}
+
+/**
+ * gda_select_alter_select_for_empty
+ *
+ * Returns: a new #GdaStatement
+ */
+GdaStatement *
+gda_select_alter_select_for_empty (GdaStatement *stmt, GError **error)
+{
+	GdaStatement *estmt;
+	GdaSqlStatement *sqlst;
+	GdaSqlStatementSelect *stsel;
+
+	g_assert (gda_statement_get_statement_type (stmt) == GDA_SQL_STATEMENT_SELECT);
+	g_object_get (G_OBJECT (stmt), "structure", &sqlst, NULL);
+	g_assert (sqlst);
+
+	if (sqlst->sql) {
+		g_free (sqlst->sql);
+		sqlst->sql = NULL;
+	}
+	stsel = (GdaSqlStatementSelect*) sqlst->contents;
+
+	/* set the WHERE condition to "1 = 0" */
+	GdaSqlExpr *expr, *cond = stsel->where_cond;
+	GdaSqlOperation *op;
+	if (cond)
+		gda_sql_expr_free (cond);
+	cond = gda_sql_expr_new (GDA_SQL_ANY_PART (stsel));
+	stsel->where_cond = cond;
+	op = gda_sql_operation_new (GDA_SQL_ANY_PART (cond));
+	cond->cond = op;
+	op->operator_type = GDA_SQL_OPERATOR_TYPE_EQ;
+	expr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
+	op->operands = g_slist_prepend (NULL, expr);
+	g_value_set_int ((expr->value = gda_value_new (G_TYPE_INT)), 1);
+	expr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
+	op->operands = g_slist_prepend (op->operands, expr);
+	g_value_set_int ((expr->value = gda_value_new (G_TYPE_INT)), 0);
+
+	/* replace any selected field which has a parameter with NULL */
+	gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stsel), (GdaSqlForeachFunc) param_to_null_foreach,
+				  NULL, NULL);
+
+	/* create new statement */
+	estmt = g_object_new (GDA_TYPE_STATEMENT, "structure", sqlst, NULL);
+	gda_sql_statement_free (sqlst);
+	return estmt;
+}
 
 /**
  * gda_server_provider_get_schema_nb_columns
@@ -418,7 +491,7 @@ gda_server_provider_init_schema_model (GdaDataModel *model, GdaConnectionSchema
 	for (i = 0; i < nbcols; i++) {
                 column = gda_data_model_describe_column (GDA_DATA_MODEL (model), i);
 
-                gda_column_set_title (column, spec[i].col_name);
+                gda_column_set_description (column, spec[i].col_name);
                 gda_column_set_name (column, spec[i].col_name);
                 gda_column_set_g_type (column, spec[i].data_type);
         }
@@ -457,10 +530,10 @@ gda_server_provider_test_schema_model (GdaDataModel *model, GdaConnectionSchema
 
                 column = gda_data_model_describe_column (GDA_DATA_MODEL (model), i);
 
-		if (strcmp (gda_column_get_title (column), spec[i].col_name)) {
+		if (strcmp (gda_column_get_description (column), spec[i].col_name)) {
 			g_set_error (error, 0, 0,
 				     _("Data model for schema has a wrong column title: '%s' instead of '%s'"),
-				     gda_column_get_title (column), spec[i].col_name);
+				     gda_column_get_description (column), spec[i].col_name);
 			return FALSE;
 		}
 
