@@ -51,7 +51,8 @@ typedef enum {
 	GDA_STATEMENT_MODEL_RANDOM_ACCESS   = 1 << 0,
 	GDA_STATEMENT_MODEL_CURSOR_FORWARD  = 1 << 1,
 	GDA_STATEMENT_MODEL_CURSOR_BACKWARD = 1 << 2,
-	GDA_STATEMENT_MODEL_CURSOR          = GDA_STATEMENT_MODEL_CURSOR_FORWARD | GDA_STATEMENT_MODEL_CURSOR_BACKWARD
+	GDA_STATEMENT_MODEL_CURSOR          = GDA_STATEMENT_MODEL_CURSOR_FORWARD | GDA_STATEMENT_MODEL_CURSOR_BACKWARD,
+	GDA_STATEMENT_MODEL_ALLOW_NOPARAM   = 1 << 3
 } GdaStatementModelUsage;
 
 typedef enum {
