@@ -61,6 +61,7 @@ gda_postgres_handler_bin_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaPostgresHandlerBinClass),
 			(GBaseInitFunc) NULL,
@@ -79,8 +80,12 @@ gda_postgres_handler_bin_get_type (void)
 			NULL
 		};
 
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaPostgresHandlerBin", &info, 0);
-		g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
+		g_static_mutex_lock (&registering);
+		if (type == 0) {
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaPostgresHandlerBin", &info, 0);
+			g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
+		}
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
