@@ -31,6 +31,8 @@ static gboolean test7 (GError **error);
 static gboolean test8 (GError **error);
 static gboolean test9 (GError **error);
 static gboolean test10 (GError **error);
+static gboolean test11 (GError **error);
+static gboolean test12 (GError **error);
 
 TestFunc tests[] = {
 	test1,
@@ -42,7 +44,9 @@ TestFunc tests[] = {
 	test7,
 	test8,
 	test9,
-	test10
+	test10,
+	test11,
+	test12
 };
 
 int 
@@ -878,6 +882,123 @@ test10 (GError **error)
 	return TRUE;
 }
 
+static gboolean
+test11 (GError **error)
+{
+	GdaHolder *h;
+	const GValue *cvalue;
+	GValue *value;
+
+	h = gda_holder_new (GDA_TYPE_NULL);
+	emitted_signals_monitor_holder (h);
+
+	/***/
+	value = gda_value_new_from_string ("my string", G_TYPE_STRING);
+	emitted_signals_reset ();
+	if (gda_holder_set_value (h, value, NULL)) {
+		g_set_error (error, 0, 0,
+			     "GdaHolder's change should have failed");
+		return FALSE;
+	}
+	gda_value_free (value);
+
+	/***/
+	if (!gda_holder_set_value (h, NULL, error))
+		return FALSE;
+	if (!emitted_signals_find (h, "changed", error))
+		return FALSE;
+	if (!emitted_signals_chech_empty (NULL, "changed", error))
+		return FALSE;
+	
+	/***/
+	g_object_set (G_OBJECT (h), "g-type", G_TYPE_STRING, NULL);
+	value = gda_value_new_from_string ("my other string", G_TYPE_STRING);
+	emitted_signals_reset ();
+	if (!gda_holder_set_value (h, value, error))
+		return FALSE;
+	if (!emitted_signals_find (h, "changed", error))
+		return FALSE;
+	if (!emitted_signals_chech_empty (NULL, "changed", error))
+		return FALSE;
+
+	/***/
+	cvalue = gda_holder_get_value (h);
+	if (!cvalue || gda_value_differ (value, cvalue)) {
+		g_set_error (error, 0, 0,
+			     "GdaHolder's value is incorrect");
+		return FALSE;
+	}
+	gda_value_free (value);
+
+	g_object_unref (h);
+
+	return TRUE;
+}
+
+static gboolean
+test12 (GError **error)
+{
+	GdaHolder *h;
+	const GValue *cvalue;
+	GValue *value;
+
+	h = gda_holder_new (GDA_TYPE_NULL);
+
+	/***/
+	value = gda_value_new_from_string ("my string", G_TYPE_STRING);
+	if (gda_holder_get_attribute (h, "attname1")) {
+		g_set_error (error, 0, 0,
+			     "gda_holder_get_attribute() should have retunred NULL");
+		return FALSE;
+	}
+	gda_holder_set_attribute (h, "attname1", value);
+	gda_value_free (value);
+	cvalue = gda_holder_get_attribute (h, "attname1");
+	if (!cvalue) {
+		g_set_error (error, 0, 0,
+			     "gda_holder_get_attribute() should have retunred a value");
+		return FALSE;
+	}
+	value = gda_value_new_from_string ("my string", G_TYPE_STRING);
+	if (gda_value_differ (cvalue, value)) {
+		g_set_error (error, 0, 0,
+			     "gda_holder_get_attribute() retunred a wrong value");
+		return FALSE;
+	}
+
+	/***/
+	GdaHolder *copy;
+	gchar *name;
+
+	g_object_set (G_OBJECT (h), "name", "thename", NULL);
+	copy = gda_holder_copy (h);
+	cvalue = gda_holder_get_attribute (copy, "attname1");
+	if (!cvalue) {
+		g_set_error (error, 0, 0,
+			     "gda_holder_get_attribute() should have retunred a value");
+		return FALSE;
+	}
+	if (gda_value_differ (cvalue, value)) {
+		g_set_error (error, 0, 0,
+			     "gda_holder_get_attribute() retunred a wrong value");
+		return FALSE;
+	}
+	gda_value_free (value);
+
+	g_object_get (G_OBJECT (copy), "name", &name, NULL);
+	if (strcmp (name, "thename")) {
+		g_set_error (error, 0, 0,
+			     "gda_holder_copy() did not copy the name");
+		return FALSE;
+	}
+	g_free (name);
+
+	g_object_unref (copy);
+	g_object_unref (h);
+
+	return TRUE;
+}
+
 
 /*
  * Signals testing
